@@ -1,9 +1,7 @@
 from web3 import Web3
 
-from .addresses import *
-
 from .types import MessageStatus
-from .utils import get_provider, load_abi, determine_direction
+from .utils import get_provider, load_abi, determine_direction, is_chain_supported, read_addresses
 
 class Contract():
 
@@ -19,19 +17,19 @@ class Contract():
     
 class OptimismPortal(Contract):
 
-    def __init__(self, account, provider=None, network="mainnet"):
+    def __init__(self, chain_id_l1, chain_id_l2, account, provider=None):
         
         if provider is None:
-            self.provider = get_provider(network=network)
+            self.provider = get_provider(chain_id_l1)
         else:
             self.provider = provider
 
-        if network == "mainnet":
-            self.address = OPTIMISM_PORTAL
-        elif network == "goerli":
-            self.address = OPTIMISM_PORTAL_GOERLI
-        elif network == "sepolia":
-            self.address = OPTIMISM_PORTAL_SEPOLIA
+        if is_chain_supported(chain_id_l1) is False:
+            raise Exception(f"Chain ID {chain_id_l1} not supported: add it to the config.json file or open a request to add it.")
+        if is_chain_supported(chain_id_l2) is False:
+            raise Exception(f"Chain ID {chain_id_l2} not supported: add it to the config.json file or open a request to add it.")
+
+        self.address = read_addresses(chain_id_l1, chain_id_l2, layer="l1")["OPTIMISM_PORTAL"]
         
         self.account = account
         self.contract = self.provider.eth.contract(address=self.address, abi=load_abi("OPTIMISM_PORTAL"))
@@ -76,43 +74,40 @@ class OptimismPortal(Contract):
 
 class StandardBridge(Contract):
     
-    def __init__(self, account, from_chain_id=1, to_chain_id=10, provider=None, network="mainnet"):
+    def __init__(self, account, from_chain_id, to_chain_id, provider=None):
         
-        l1_to_l2 = determine_direction(from_chain_id, to_chain_id)
-        l2 = not l1_to_l2
+        self.l1_to_l2 = determine_direction(from_chain_id, to_chain_id)
 
         if provider is None:
-            self.provider = get_provider(l2=l2, network=network)
+            self.provider = get_provider(from_chain_id)
         else:
             self.provider = provider
 
-        if l1_to_l2:
-            if network == "mainnet":
-                self.address = L1_STANDARD_BRIDGE
-            elif network == "goerli":
-                self.address = L1_STANDARD_BRIDGE_GOERLI
-            elif network == "sepolia":
-                self.address = L1_STANDARD_BRIDGE_SEPOLIA
+        if is_chain_supported(from_chain_id) is False:
+            raise Exception(f"Origin Chain ID {from_chain_id} not supported: add it to the config.json file or open a request to add it.")
+        if is_chain_supported(to_chain_id) is False:
+            raise Exception(f"Destination Chain ID {to_chain_id} not supported: add it to the config.json file or open a request to add it.")
+
+        if self.l1_to_l2:
+            self.address = read_addresses(from_chain_id, to_chain_id, layer="l1")["L1_STANDARD_BRIDGE"]
         else:
-            if network == "mainnet":
-                self.address = L2_STANDARD_BRIDGE
-            elif network == "goerli":
-                self.address = L2_STANDARD_BRIDGE_GOERLI
-            elif network == "sepolia":
-                self.address = L2_STANDARD_BRIDGE_SEPOLIA
+            self.address = read_addresses(to_chain_id, from_chain_id, layer="l2")["L2_STANDARD_BRIDGE"]
         
         self.from_chain_id = from_chain_id
         self.to_chain_id = to_chain_id
 
         self.account = account
 
-        if l1_to_l2:
+        if self.l1_to_l2:
             self.contract = self.provider.eth.contract(address=self.address, abi=load_abi("L1_STANDARD_BRIDGE"))
         else:
             self.contract = self.provider.eth.contract(address=self.address, abi=load_abi("L2_STANDARD_BRIDGE"))
     
-    def deposit_eth_to(self, to, gas_limit, extra_data, value):
-        
+    def deposit_eth_to(self, to, value, gas_limit, extra_data):
+
+        if not self.l1_to_l2:
+            raise Exception("This method can only be called on a L1 to L2 Bridge")
+
         deposit_eth_to_tx = self.contract.functions.depositETHTo(to, gas_limit, extra_data).build_transaction({
             "from": self.account.address,
             "gas": 500000,
@@ -122,54 +117,95 @@ class StandardBridge(Contract):
 
         return self.sign_and_broadcast(deposit_eth_to_tx)
 
-    def deposit_erc20():
-        raise NotImplementedError
-
-    def withdraw_eth_to(self, to, amount, gas_limit, extra_data):
+    def deposit_erc20(self, l1_token_address, l2_token_address, value, gas_limit, extra_data):
         
-        withdraw_eth_to_tx = self.contract.functions.withdrawTo("0xDeadDeAddeAddEAddeadDEaDDEAdDeaDDeAD0000", to, amount, gas_limit, extra_data).build_transaction({
+        if not self.l1_to_l2:
+            raise Exception("This method can only be called on a L1 to L2 Bridge")
+        
+        deposit_erc20_tx = self.contract.functions.depositERC20(l1_token_address, l2_token_address, value, gas_limit, extra_data).build_transaction({
+            "from": self.account.address,
+            "gas": 700000,
+            "nonce": self.provider.eth.get_transaction_count(self.account.address)
+        })
+
+        return self.sign_and_broadcast(deposit_erc20_tx)
+        
+    def deposit_erc20_to(self, l1_token_address, l2_token_address, to, value, gas_limit, extra_data):
+        
+        if not self.l1_to_l2:
+            raise Exception("This method can only be called on a L1 to L2 Bridge")
+        
+        deposit_erc20_to_tx = self.contract.functions.depositERC20To(l1_token_address, l2_token_address, to, value, gas_limit, extra_data).build_transaction({
+            "from": self.account.address,
+            "gas": 700000,
+            "nonce": self.provider.eth.get_transaction_count(self.account.address)
+        })
+
+        return self.sign_and_broadcast(deposit_erc20_to_tx)
+        
+
+    def withdraw_eth_to(self, to, value, gas_limit, extra_data):
+
+        if self.l1_to_l2:
+            raise Exception("This method can only be called on a L2 to L1 Bridge")
+        
+        withdraw_eth_to_tx = self.contract.functions.withdrawTo("0xDeadDeAddeAddEAddeadDEaDDEAdDeaDDeAD0000", to, value, gas_limit, extra_data).build_transaction({
             "from": self.account.address,
             "gas": 500000,
             "nonce": self.provider.eth.get_transaction_count(self.account.address),
-            "value": amount
+            "value": value
         })
 
         return self.sign_and_broadcast(withdraw_eth_to_tx)
+    
+    def bridge_erc20(self, l1_token_address, l2_token_address, value, gas_limit, extra_data):
 
-    def withdraw_erc20_to():
-        raise NotImplementedError
+        if self.l1_to_l2:
+            raise Exception("This method can only be called on a L2 to L1 Bridge")
+        
+        withdraw_erc20_tx = self.contract.functions.bridgeERC20(l2_token_address, l1_token_address, value, gas_limit, extra_data).build_transaction({
+            "from": self.account.address,
+            "gas": 500000,
+            "nonce": self.provider.eth.get_transaction_count(self.account.address)
+        })
+
+        return self.sign_and_broadcast(withdraw_erc20_tx)
+
+    def bridge_erc20_to(self, l1_token_address, l2_token_address, to, value, gas_limit, extra_data):
+        
+        if self.l1_to_l2:
+            raise Exception("This method can only be called on a L2 to L1 Bridge")
+        
+        withdraw_erc20_to_tx = self.contract.functions.bridgeERC20To(l2_token_address, l1_token_address, to, value, gas_limit, extra_data).build_transaction({
+            "from": self.account.address,
+            "gas": 500000,
+            "nonce": self.provider.eth.get_transaction_count(self.account.address)
+        })
 
 class CrossChainMessengerContract(Contract):
 
-    def __init__(self, account, from_chain_id=1, to_chain_id=10, provider=None, network="mainnet"):
+    def __init__(self, account, from_chain_id=1, to_chain_id=10, provider=None):
 
         self.l1_to_l2 = determine_direction(from_chain_id, to_chain_id)
-        l2 = not self.l1_to_l2
 
         if provider is None:
-            self.provider = get_provider(l2=l2, network=network)
+            self.provider = get_provider(chain_id=from_chain_id)
         else:
             self.provider = provider
 
+        if is_chain_supported(from_chain_id) is False:
+            raise Exception(f"Origin Chain ID {from_chain_id} not supported: add it to the config.json file or open a request to add it.")
+        if is_chain_supported(to_chain_id) is False:
+            raise Exception(f"Destination Chain ID {to_chain_id} not supported: add it to the config.json file or open a request to add it.")
+
         if self.l1_to_l2:
-            if network == "mainnet":
-                self.address = L1_CROSS_CHAIN_MESSENGER
-            elif network == "goerli":
-                self.address = L1_CROSS_CHAIN_MESSENGER_GOERLI
-            elif network == "sepolia":
-                self.address = L1_CROSS_CHAIN_MESSENGER_SEPOLIA
+            self.address = read_addresses(from_chain_id, to_chain_id, layer="l1")["L1_CROSS_CHAIN_MESSENGER"]
         else:
-            if network == "mainnet":
-                self.address = L2_CROSS_CHAIN_MESSENGER
-            elif network == "goerli":
-                self.address = L2_CROSS_CHAIN_MESSENGER_GOERLI
-            elif network == "sepolia":
-                self.address = L2_CROSS_CHAIN_MESSENGER_SEPOLIA
+            self.address = read_addresses(to_chain_id, from_chain_id, layer="l2")["L2_CROSS_CHAIN_MESSENGER"]
         
         self.from_chain_id = from_chain_id
         self.to_chain_id = to_chain_id
 
-        self.network = network
         self.account = account
 
         if self.l1_to_l2:
@@ -196,19 +232,19 @@ class CrossChainMessengerContract(Contract):
     
 class L2OutputOracle():
 
-    def __init__(self, account, provider=None, network="mainnet"):
+    def __init__(self, chain_id_l1, chain_id_l2, account, provider=None):
             
         if provider is None:
-            self.provider = get_provider(network=network)
+            self.provider = get_provider(chain_id_l1)
         else:
             self.provider = provider
 
-        if network == "mainnet":
-            self.address = L2_OUTPUT_ORACLE
-        elif network == "goerli":
-            self.address = L2_OUTPUT_ORACLE_GOERLI
-        elif network == "sepolia":
-            self.address = L2_OUTPUT_ORACLE_SEPOLIA
+        if is_chain_supported(chain_id_l1) is False:
+            raise Exception(f"Chain ID {chain_id_l1} not supported: add it to the config.json file or open a request to add it.")
+        if is_chain_supported(chain_id_l2) is False:
+            raise Exception(f"Chain ID {chain_id_l2} not supported: add it to the config.json file or open a request to add it.")
+    
+        self.address = read_addresses(chain_id_l1, chain_id_l2, layer="l1")["L2_OUTPUT_ORACLE"]
 
         self.account = account
 
@@ -244,19 +280,14 @@ class L2OutputOracle():
     
 class L2ToL1MessagePasser(Contract):
 
-    def __init__(self, account, provider=None, network="mainnet"):
+    def __init__(self, chain_id_l1, chain_id_l2, account, provider=None):
 
         if provider is None:
-            self.provider = get_provider(l2=True, network=network)
+            self.provider = get_provider(chain_id_l2)
         else:
             self.provider = provider
 
-        if network == "mainnet":
-            self.address = L2_TO_L1_MESSAGE_PASSER
-        elif network == "goerli":
-            self.address = L2_TO_L1_MESSAGE_PASSER_GOERLI
-        elif network == "sepolia":
-            self.address = L2_TO_L1_MESSAGE_PASSER_SEPOLIA
+        self.address = read_addresses(chain_id_l1, chain_id_l2, layer="l2")["L2_TO_L1_MESSAGE_PASSER"]
 
         self.account = account
 
